package org.groove;

import org.apache.deltaspike.cdise.api.CdiContainer;
import org.apache.deltaspike.cdise.api.CdiContainerLoader;
import org.apache.deltaspike.core.api.provider.BeanProvider;
import org.junit.After;
import org.junit.Before;

import de.groovie.test.cdi00.common.data.CalcParmIn;
import de.groovie.test.cdi00.common.data.CalcParmOut;
import de.groovie.test.cdi00.interfaces.svc.CalculationEngineIf;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Unit test for simple App.
 */
public class AppTest 
    extends TestCase
{
	
	/*
	 * ~~ CDI related parameters ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
	 */
	private CdiContainer 		cdiContainer  = null ;
	private CalculationEngineIf calculateBilling ;
	
	
    /**
     * Create the test case
     *
     * @param testName name of the test case
     */
    public AppTest( String testName )
    {
        super( testName );
    }

    /**
     * @return the suite of tests being tested
     */
    public static Test suite()
    {
        return new TestSuite( AppTest.class );
    }
   
    
    /*
     * Junit helpers
     */
	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		
		/*
		 * Setup CDI container 
		 */
        cdiContainer = CdiContainerLoader.getCdiContainer();
        cdiContainer.boot();
        cdiContainer.getContextControl().startContexts();
        
        /*
         * programatic injection
         */
        calculateBilling =  BeanProvider.getContextualReference(CalculationEngineIf.class, false); 
        
        if (null == calculateBilling) {
        	System.out.print("uffz") ;
        }
	}

    
	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {

		/*
		 * shutdown the CDI container 
		 */
		if (null != cdiContainer) {
			cdiContainer.shutdown() ;
		}
		calculateBilling = null  ;
	}
	
    
    /*
     * ~~ This is the test  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     */
       
    /**
     * Rigourous Test :-)
     */
    public void testApp()
    {
    	/* 1) check if implementation could be installed */
        assertNotNull(calculateBilling) ;
        
        /* 2)  pseudo initialize the data  */
        final Long myKey = calculateBilling.regNewAge(26) ;
        	
        /* 3)  pseudo initialize the data  */
        final CalcParmIn	inParam = new CalcParmIn() ;
        inParam.setCustId(myKey) ;
        inParam.setNumber2(2) ;
        
        final float			expected = 26.0f / 2.0f ; 
        final CalcParmOut	outParam = calculateBilling.computeSomewhat(inParam) ;
        
        assertEquals(expected, outParam.getResultNum()) ;
    }
}
